	jp _GRAPHICS_END
;---------------------------------;
; Set drawing cursor coodrinates  ;
;---------------------------------;
; Inputs:                         ;
;	HL	    x-coordinate  ;
;	DE	    y-coordinate  ;
;---------------------------------;
;Note:                            ;
;	(0,0) is moved to the top ;
; 	left corner of the window ;
;---------------------------------;
;setCoordinates:
;
;	push af
;
;	ld bc, (_window_x1)
;	add hl, bc
;	ld a, $21
;	call setLCDRegister
;
;	ld h, d
;	ld l, e
;	ld bc, (_window_y1)
;	add hl, bc
;	ld a, $20
;	call setLCDRegister
;
;	pop af
;
;	ret
;
;---------------------------------;
; Reset cursor to (0,0).          ;
;---------------------------------;
;Note:                            ;
;	Called automatically      ;
;	after setWindow           ;
;---------------------------------;
resetCursor:

	push af

	ld hl, (_window_x1)
	ld a, $21
	call setLCDRegister

	ld hl, (_window_y1)
	ld a, $20
	call setLCDRegister

	pop af

	ret

;---------------------------------;
; Print pixel to the screen       ;
;---------------------------------;
; Inputs:                         ;
;	HL	color             ;
;---------------------------------;
setPixel:

	push af
	ld a, $22
	call setLCDRegister
	pop af

	ret


;---------------------------------;
; Sets screen mode                ;
;---------------------------------;
; Inputs:                         ;
;	_window_x1                ;
;	_window_y1                ;
;	_window_x2                ;
;	_window_y2                ;
;---------------------------------;
; Note: HL is modified            ;
;---------------------------------;
setWindow:
	push af
	
	ld a, $52
	ld hl, (_window_x1)
	call setLCDRegister

	ld a, $50
	ld hl, (_window_y1)
	call setLCDRegister

	ld a, $53
	ld hl, (_window_x2)
	call setLCDRegister

	ld a, $51
	ld hl, (_window_y2)
	call setLCDRegister
	
	pop af

	call resetCursor 
	call calculateDimensions

	ret 

;---------------------------------;
; Calculate dimensions for window ;
;---------------------------------;
; Note: Do not call this.         ;
;	It is automatically       ;
;	called by setWindow.      ;
;---------------------------------;
calculateDimensions:

	push hl 

	;calculate width
	ld a, 0	; sbc also substracts the C flag
	cp 0	; so using this we can reset it
	ld hl, (_window_x1)
	ld b, h
	ld c, l
	ld hl, (_window_x2)
	sbc hl, bc
	inc hl

	;store width for later
	ld (_window_width), hl

	;calculate height
	ld a, 0	; sbc also substracts the C flag
	cp 0	; so using this we can reset it
	ld hl, (_window_y1)
	ld b, h
	ld c, l
	ld hl, (_window_y2)
	sbc hl, bc
	inc hl
	ld (_window_height), hl

	;bc holds height
	;hl and de holds width
	ld b, h
	ld c, l
	ld de, (_window_width)
	ld hl, (_window_width)

	;add hl to hl bc times
	;essentially hl = bc*hl
	;then set bc = hl
	;then set hl = color code
	dec bc
_CalculateDimensions_calculate_size:
	add hl, de
	dec bc
	ld a, b
	cp 0
	jp nz, _CalculateDimensions_calculate_size
	ld a, c
	cp 0
	jp nz, _CalculateDimensions_calculate_size

	ld (_window_size), hl

	pop hl
	ret

;user-set
_window_x1:
	.db 0,0
_window_y1:
	.db 0,0
_window_x2:
	.db 0,0
_window_y2:
	.db 0,0
_window_foreground:
	.db 0,0
_window_background:
	.db 0,0
;library-set (do not modify)
_window_width:
	.db 0,0
_window_height:
	.db 0,0
_window_size:
	.db 0,0
_window_sprite_addr:
	.db 0,0


;---------------------------------;
; Fills window with a color       ;
;---------------------------------;
; Inputs:                         ;
;	HL	color to fill     ;
;---------------------------------;
; Notes:                          ;
;	This only works on        ;
;	windows with a            ;
;       width*height less than    ;
;	2^16.                     ;
;---------------------------------;
fillWindow:
	push af

	ld bc, (_window_size)
	;draw the color code bc times
_fillWindow_draw:
	call setPixel 
	dec bc
	ld a, b
	cp 0
	jp nz, _fillWindow_draw
	ld a, c
	cp 0
	jp nz, _fillWindow_draw
	ld b, h
	ld c, l

	pop af

	ret

;Commented out due to memory constraints 
;---------------------------------;
; Draws a sprite to the screen    ;
;---------------------------------;
; Inputs:                         ;
;	HL	image data        ;
;---------------------------------;
; Notes:                          ;
;	This only works on        ;
;	windows with a            ;
;       width*height less than    ;
;	2^16.                     ;
;---------------------------------;
;drawSprite:
;	push af
;
;	ld (_window_sprite_addr), hl
;	ld bc, (_window_size)
;	;draw the color code bc times
;_drawSprite_draw:
;	ld a, 0
;	cp 0
;	ld hl, (_window_size)
;	sbc hl, bc
;	ld d, h
;	ld e, l
;	add hl, de
;	ld de, (_window_sprite_addr)
;	add hl, de
;	ld a, (hl)
;	ld d, a
;	inc hl
;	ld a, (hl)
;	ld e, a
;	ld h, d
;	ld l, e
;
;	call setPixel 
;	dec bc
;	ld a, b
;	cp 0
;	jp nz, _drawSprite_draw
;	ld a, c
;	cp 0
;	jp nz, _drawSprite_draw
;	ld b, h
;	ld c, l
;
;	pop af
;
;	ret


;---------------------------------;
; Draws a sprite to the screen    ;
;---------------------------------;
; Inputs:                         ;
;	HL	image data        ;
;---------------------------------;
; Notes:                          ;
;	This is identical to      ;
;	drawSprite, however, all  ;
;	$FFFF pixels are replaced ;
;	by _window_foreground and ;
;	all non-$FFFF colors are  ;
;	replaced by               ;
;	_window_background.       ;
;---------------------------------;
drawSpriteFGBG:
	push af

	ld (_window_sprite_addr), hl
	ld bc, (_window_size)
	;draw the color code bc times
_drawSpriteFGBG_draw:
	ld a, 0
	cp 0
	ld hl, (_window_size)
	sbc hl, bc
	ld d, h
	ld e, l
	add hl, de
	ld de, (_window_sprite_addr)
	add hl, de
	ld a, (hl)
	ld d, a
	inc hl
	ld a, (hl)
	ld e, a
	ld h, d
	ld l, e

	ld a, h
	cp $FF
	call z, _drawSpriteFGBG_getFG
	call nz, _drawSpriteFGBG_getBG

	call setPixel 
	dec bc
	ld a, b
	cp 0
	jp nz, _drawSpriteFGBG_draw
	ld a, c
	cp 0
	jp nz, _drawSpriteFGBG_draw
	ld b, h
	ld c, l

	pop af
_drawSpriteFGBG_getFG:
	ld hl, (_window_foreground)
	ret
_drawSpriteFGBG_getBG:
	ld hl, (_window_background)
	ret
;---------------------------------;
; Fill screen with a color        ;
;---------------------------------;
; Inputs:                         ;
;	HL	color to fill     ;
;---------------------------------;
fillScreen:
	push hl
	ld hl, 0
	ld (_window_x1), hl
	ld hl, 319
	ld (_window_x2), hl
	ld hl, 0
	ld (_window_y1), hl
	ld hl, 120
	ld (_window_y2), hl
	call setWindow
	pop hl
	call fillWindow

	push hl
	ld hl, 0
	ld (_window_x1), hl
	ld hl, 319
	ld (_window_x2), hl
	ld hl, 121
	ld (_window_y1), hl
	ld hl, 239
	ld (_window_y2), hl
	call setWindow
	pop hl
	call fillWindow

	ret

;---------------------------------;
; Sets screen mode to full window ;
;---------------------------------;
; Inputs:                         ;
;	<None>                    ;
;---------------------------------;
setWindowFull:
	push af
	push hl

	;set the y-minimum register to 0 
	ld a, $50
	ld hl, 0
	call setLCDRegister

	;set the y-maximum register to 239
	ld a, $51
	ld hl, 239
	call setLCDRegister

	;set the x-minimum register to 0
	ld a, $52
	ld hl, 0
	call setLCDRegister

	;set the x-maximum register to 0
	ld a, $53
	ld hl, 319
	call setLCDRegister

	pop hl
	pop af

	call resetCursor
	call calculateDimensions 

	ret


;---------------------------------;
; Sets a value to an LCD register ;
;---------------------------------;
; Inputs:                         ;
;	A	Register ID       ;
;	HL	Register value    ;
;---------------------------------;
setLCDRegister:
	push af
	out ($10), a
	out ($10), a
	ld a, h
	out ($11),a
	ld a, l
	out ($11), a
	pop af
	ret

;---------------------------------;
; Sets the CPU to 15MHz           ;
;---------------------------------;
; Inputs:                         ;
;	<none>                    ;
;---------------------------------;
setCPUFast:
	push af
	ld a, 1
	out ($20), a
	pop af
	ret

;---------------------------------;
; Sets the CPU to 6MHz            ;
;---------------------------------;
; Inputs:                         ;
;	<none>                    ;
;---------------------------------;
;setCPUSlow:
;	push af
;	ld a, 0
;	out ($20), a
;	pop af
;	ret

_GRAPHICS_END:
